@@ -1,5 +1,5 @@
 """
-Collection of functions for accelerator physics for PhD projects 
+Collection of functions for accelerator physics for PhD projects  - local test version (when tested, merge with other version)
 
 Author: Elias Waagaard, elias.walter.waagaard@cern.ch
 """
@@ -10,7 +10,7 @@ import matplotlib as mpl
 from scipy.fft import fft, fftfreq
 import NAFFlib # Numerical Analysis of the Fundamental Frequencies 
 
-# Initiate class containing all the necessary functions for plotting 
+# Initiate class containing helpful functions for plotting 
 class plot_tools: 
         
     def plot_twiss(fig, twiss, twiss_from_madx=False, plot_magnets=False, also_closed_orbit=False):
@@ -91,45 +91,71 @@ class plot_tools:
         fig.tight_layout()
         
 
-    def plot_phase_space_ellipse(fig, tracker, axis='horizontal'):
+    def plot_phase_space_ellipse(fig, tracker, axis='both'):
         """
         Method to plot phase space ellipse from X-suite tracking 
         """
+        mpl.rcParams['axes.labelsize'] = 20
+        mpl.rcParams['xtick.labelsize'] = 14
+        mpl.rcParams['ytick.labelsize'] = 14
+        
         fig.suptitle('Phase space ellipse',fontsize=18)
-        ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
-        ax.yaxis.label.set_size(20)
-        ax.xaxis.label.set_size(20)
-        plt.xticks(fontsize=14)  
-        plt.yticks(fontsize=14)
-        if axis == 'horizontal':
+        if axis == 'both':
+            ax = fig.add_subplot(2, 1, 1)  # create an axes object in the figure
             ax.plot(tracker.record_last_track.x, tracker.record_last_track.px, 'ro', markersize=0.2, alpha=0.3)
             ax.set_ylabel("$p_{x}$")
             ax.set_xlabel("$x$")
-        if axis == 'vertical':
-            ax.plot(tracker.record_last_track.y, tracker.record_last_track.py, 'bo', markersize=0.2, alpha=0.3)
-            ax.set_ylabel("$p_{y}$")
-            ax.set_xlabel("$y$")
+            ax2 = fig.add_subplot(2, 1, 2, sharex=ax)  # create a second axes object in the figure
+            ax2.plot(tracker.record_last_track.y, tracker.record_last_track.py, 'bo', markersize=0.2, alpha=0.3)
+            ax2.set_ylabel("$p_{y}$")
+            ax2.set_xlabel("$y$")
+        else:
+            ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
+            if axis == 'horizontal':
+                ax.plot(tracker.record_last_track.x, tracker.record_last_track.px, 'ro', markersize=0.2, alpha=0.3)
+                ax.set_ylabel("$p_{x}$")
+                ax.set_xlabel("$x$")
+            if axis == 'vertical':
+                ax.plot(tracker.record_last_track.y, tracker.record_last_track.py, 'bo', markersize=0.2, alpha=0.3)
+                ax.set_ylabel("$p_{y}$")
+                ax.set_xlabel("$y$")
         fig.tight_layout()
             
             
-    def plot_centroid_motion(fig, tracker, axis='horizontal'):
+    def plot_centroid_motion(fig, tracker, axis='both'):
         """
         Method to plot centroid from tracker, either in horizontal or vertical plane 
         """
+        mpl.rcParams['axes.labelsize'] = 20
+        mpl.rcParams['xtick.labelsize'] = 14
+        mpl.rcParams['ytick.labelsize'] = 14
+        
         fig.suptitle('X-suite tracking - centroid motion',fontsize=18)
-        ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
-        ax.yaxis.label.set_size(20)
-        ax.xaxis.label.set_size(20)
-        plt.xticks(fontsize=14)  
-        plt.yticks(fontsize=14)
-        if axis == 'horizontal':
+        
+        if axis == 'both':
+            ax = fig.add_subplot(2, 1, 1)  # create an axes object in the figure
             ax.plot(np.mean(tracker.record_last_track.x, axis=0), marker='o', color='r', markersize=5)
-            ax.set_ylabel("Horizontal centroid $X$ [m]")
-            ax.set_xlabel("#turns")
-        if axis == 'vertical':
-            ax.plot(np.mean(tracker.record_last_track.y, axis=0), marker='o', color='b', markersize=5)
-            ax.set_ylabel("Vertical centroid $Y$ [m]")
+            ax.set_ylabel("Centroid $X$ [m]")
             ax.set_xlabel("#turns")
+            ax2 = fig.add_subplot(2, 1, 2, sharex=ax)  # create a second axes object in the figure
+            ax2.plot(np.mean(tracker.record_last_track.y, axis=0), marker='o', color='b', markersize=5)
+            ax2.set_ylabel("Centroid $Y$ [m]")
+            ax2.set_xlabel("#turns")
+            plt.setp(ax.get_xticklabels(), visible=False)
+        else:
+            ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
+            #ax.yaxis.label.set_size(20)
+            #ax.xaxis.label.set_size(20)
+            #plt.xticks(fontsize=14)  
+            #plt.yticks(fontsize=14)
+            if axis == 'horizontal':
+                ax.plot(np.mean(tracker.record_last_track.x, axis=0), marker='o', color='r', markersize=5)
+                ax.set_ylabel("Horizontal centroid $X$ [m]")
+                ax.set_xlabel("#turns")
+            if axis == 'vertical':
+                ax.plot(np.mean(tracker.record_last_track.y, axis=0), marker='o', color='b', markersize=5)
+                ax.set_ylabel("Vertical centroid $Y$ [m]")
+                ax.set_xlabel("#turns")
         fig.tight_layout()
             
             
@@ -188,6 +214,184 @@ class plot_tools:
         
         return Q_x, Q_y, ax
     
+
+class resonance_lines(object):
+    """
+    Class from Foteini Asvesta to plot resonance lines of chosen orders in tune diagram
+    Provide input of ranges in Qx and Qy, the orders and the periodiciyu of the resonances
+    """    
+    def __init__(self, Qx_range, Qy_range, orders, periodicity):
+        
+        # Define plotting parameters
+        mpl.rcParams['axes.labelsize'] = 20
+        mpl.rcParams['xtick.labelsize'] = 14
+        mpl.rcParams['ytick.labelsize'] = 14
+        
+        if np.std(Qx_range):
+              self.Qx_min = np.min(Qx_range)
+              self.Qx_max = np.max(Qx_range)
+        else:
+              self.Qx_min = np.floor(Qx_range)-0.05
+              self.Qx_max = np.floor(Qx_range)+1.05
+        if np.std(Qy_range):
+              self.Qy_min = np.min(Qy_range)
+              self.Qy_max = np.max(Qy_range)
+        else:
+              self.Qy_min = np.floor(Qy_range)-0.05
+              self.Qy_max = np.floor(Qy_range)+1.05
+  
+        self.periodicity = periodicity
+                                      
+        nx, ny = [], []
+  
+        for order in np.nditer(np.array(orders)):
+              t = np.array(range(-order, order+1))
+              nx.extend(order - np.abs(t))
+              ny.extend(t)
+        nx = np.array(nx)
+        ny = np.array(ny)
+      
+        cextr = np.array([nx*np.floor(self.Qx_min)+ny*np.floor(self.Qy_min), \
+                            nx*np.ceil(self.Qx_max)+ny*np.floor(self.Qy_min), \
+                            nx*np.floor(self.Qx_min)+ny*np.ceil(self.Qy_max), \
+                            nx*np.ceil(self.Qx_max)+ny*np.ceil(self.Qy_max)], dtype='int')
+        cmin = np.min(cextr, axis=0)
+        cmax = np.max(cextr, axis=0)
+        res_sum = [range(cmin[i], cmax[i]+1) for i in range(cextr.shape[1])]                                
+        self.resonance_list = zip(nx, ny, res_sum)
+        
+    def plot_resonance(self, figure_object = None, interactive=True):    
+        if(interactive): 
+            plt.ion()
+        if figure_object:
+            fig = figure_object
+            plt.figure(fig.number)
+        else:
+            fig = plt.figure()
+        Qx_min = self.Qx_min
+        Qx_max = self.Qx_max
+        Qy_min = self.Qy_min
+        Qy_max = self.Qy_max 
+        plt.xlim(Qx_min, Qx_max)
+        plt.ylim(Qy_min, Qy_max)
+        plt.xlabel('$\mathrm{Q_x}$' , fontsize='large')
+        plt.ylabel('$\mathrm{Q_y}$', fontsize='large')        
+        for resonance in self.resonance_list:
+            nx = resonance[0]
+            ny = resonance[1]
+            for res_sum in resonance[2]:        
+                if ny:
+                    line, = plt.plot([Qx_min, Qx_max], \
+                        [(res_sum-nx*Qx_min)/ny, (res_sum-nx*Qx_max)/ny])
+                else:
+                    line, = plt.plot([np.float(res_sum)/nx, np.float(res_sum)/nx],[Qy_min, Qy_max])
+                if ny%2:
+                    plt.setp(line, linestyle='--', zorder=1) # for skew resonances
+                if res_sum%self.periodicity:
+                    plt.setp(line, color='b', zorder=1)    # non-systematic resonances
+                else:
+                    plt.setp(line, color='r', zorder=1, linewidth=2.0) # systematic resonances
+        if(interactive):
+            plt.draw()
+        return fig
+    
+    def plot_resonance_and_tune_footprint(self, tracker, Q_work_int = None, figure_object = None, interactive=False):    
+        """
+        Method to plot tune footprint and resonances in the same plot 
+        """
+        if(interactive): 
+            plt.ion()
+        if figure_object:
+            fig = figure_object
+            plt.figure(fig.number)
+        else:
+            fig = plt.figure()
+        ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
+        
+        Qx_min = self.Qx_min
+        Qx_max = self.Qx_max
+        Qy_min = self.Qy_min
+        Qy_max = self.Qy_max 
+        ax.set_xlabel('$\mathrm{Q_x}$')
+        ax.set_ylabel('$\mathrm{Q_y}$')        
+        ax.set_xlim(Qx_min, Qx_max)
+        ax.set_ylim(Qy_min, Qy_max)
+        for resonance in self.resonance_list:
+            nx = resonance[0]
+            ny = resonance[1]
+            for res_sum in resonance[2]:        
+                if ny:
+                    line, = ax.plot([Qx_min, Qx_max], \
+                        [(res_sum-nx*Qx_min)/ny, (res_sum-nx*Qx_max)/ny])
+                else:
+                    line, = ax.plot([np.float(res_sum)/nx, np.float(res_sum)/nx],[Qy_min, Qy_max])
+                if ny%2:
+                    plt.setp(line, linestyle='--', zorder=1) # for skew resonances
+                if res_sum%self.periodicity:
+                    plt.setp(line, color='b', zorder=1)    # non-systematic resonances
+                else:
+                    plt.setp(line, color='r', zorder=1, linewidth=2.0) # systematic resonances
+        if(interactive):
+            plt.draw()
+            
+        # Get the fractional tunes from NAFF
+        Q_x = np.zeros(len(tracker.record_last_track.x))    
+        Q_y = np.zeros(len(tracker.record_last_track.y))
+        
+        # Iterate over turn-by-turn data to find horizontal and vertical tune of each particle
+        for count, particle in enumerate(tracker.record_last_track.x):
+            Q_x[count] = NAFFlib.get_tune(particle) 
+            
+        for count, particle in enumerate(tracker.record_last_track.y):
+            Q_y[count] = NAFFlib.get_tune(particle) 
+            
+        # Add integer tune to fractional tune 
+        if Q_work_int is not None:
+            Q_x += Q_work_int
+            Q_y += Q_work_int
+                        
+        ax.plot(Q_x, Q_y, 'go', markersize=6.5, alpha=0.3)
+            
+        return ax, Q_x, Q_y
+    
+        
+    def plot_tune_footprint_from_tracker(self, fig, tracker, int_Q):
+        """
+        Method to find tune of all particles using NAFF (Numerical Analysis of the Fundamental Frequencies)
+        
+        Also returns min and max values for the tune footprints
+        """
+        Q_x = np.zeros(len(tracker.record_last_track.x))    
+        Q_y = np.zeros(len(tracker.record_last_track.y))
+        
+        # Iterate over turn-by-turn data to find horizontal and vertical tune of each particle
+        for count, particle in enumerate(tracker.record_last_track.x):
+            Q_x[count] = NAFFlib.get_tune(particle) 
+            
+        for count, particle in enumerate(tracker.record_last_track.y):
+            Q_y[count] = NAFFlib.get_tune(particle) 
+        
+        # Add integer tune to fractional tune 
+        Q_x_full = Q_x+int_Q
+        Q_y_full = Q_y+int_Q
+        
+        ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
+        #ax.yaxis.label.set_size(20)
+        #ax.xaxis.label.set_size(20)
+        #plt.xticks(fontsize=14)  
+        #plt.yticks(fontsize=14)
+        ax.plot(Q_x_full, Q_y_full, 'go', markersize=3.5, alpha=0.3)
+        Qx_min, Qx_max, Qy_min, Qy_max = np.min(Q_x), np.max(Q_x), np.min(Q_y), np.max(Q_y)
+        
+        return Qx_min, Qx_max, Qy_min, Qy_max
+    
+    def print_resonances(self):
+        for resonance in self.resonance_list:
+            for res_sum in resonance[2]:
+                print(str(resonance[0]).rjust(3), 'Qx ', ("+", "-")[resonance[1]<0], \
+                      str(abs(resonance[1])).rjust(2), 'Qy = ', str(res_sum).rjust(3), \
+                      '\t', ("(non-systematic)", "(systematic)")[res_sum%self.periodicity==0])    
+
 # Descripte class of particles to find properties 
 class particles:
     
